@@ -6,6 +6,7 @@ use warnings;
 
 use Moose;
 use Class::Load 0.20 qw(load_class);
+use Data::Dump 'dump';
 use MooseX::Aliases;
 use MooseX::StrictConstructor;
 use Try::Tiny;
@@ -15,7 +16,7 @@ use MooseX::Types::Moose qw(Bool HashRef Int Str);
 use MooseX::Types::Common::String qw(NonEmptySimpleStr);
 
 # ABSTRACT:  CyberSource Client object  for Business::OnlinePayment::CyberSource
-our $VERSION = '3.000004'; # VERSION
+our $VERSION = '3.000005'; # VERSION
 
 #### Subroutine Definitions ####
 
@@ -61,6 +62,10 @@ sub _authorize          {
 
 	Exception::Base->throw( $message ) if $message;
 
+	unless ( $self->require_avs() ) {
+		$data->{business_rules} = { ignore_avs_result => 1 };
+	}
+
 	my $request         = try {
 		load_class( $class )->new( $data );
 	}
@@ -76,30 +81,29 @@ sub _authorize          {
 
 	try {
 		my $response        = $self->run_transaction( $request );
+		my $res           = $response->trace->response();
 
 		if ( $response->is_success() ) {
-			my $res           = $response->trace->response();
-
 			$success          = 1;
 
 			$self->is_success( $success );
-			$self->avs_code( $response->avs_code() );
 			$self->authorization( $response->auth_code() );
-			$self->order_number( $response->request_id() );
-			$self->response_code( $res->code() );
-			$self->response_page( $res->content() );
-			$self->response_headers({
-					map { ## no critic ( BuiltinFunctions::ProhibitVoidMap )
-						$_ => $res->headers->header( $_ )
-					} $res->headers->header_field_names()
-				} );
-			$self->result_code( $response->reason_code() );
-
 			$self->cvv2_response( $response->cv_code() ) if $response->has_cv_code();
 		}
 		else {
 			$self->set_error_message( $response->reason_text() );
 		}
+
+		$self->avs_code( $response->avs_code() );
+		$self->order_number( $response->request_id() );
+		$self->response_code( $res->code() );
+		$self->response_page( $res->content() );
+		$self->response_headers({
+				map { ## no critic ( BuiltinFunctions::ProhibitVoidMap )
+					$_ => $res->headers->header( $_ )
+				} $res->headers->header_field_names()
+			} );
+		$self->result_code( $response->reason_code() );
 	}
 	catch {
 		$message = shift;
@@ -148,26 +152,26 @@ sub capture            {
 
 	try {
 		my $response      = $self->run_transaction( $request );
+		my $res         = $response->trace->response();
 
 		if ( $response->is_success() ) {
-			my $res         = $response->trace->response();
-
 			$success        = 1;
 
 			$self->is_success ( $success );
-			$self->order_number( $response->request_id() );
-			$self->response_code( $response->reason_code() );
-			$self->response_page( $res->content() );
-			$self->response_headers({
-					map { ## no critic ( BuiltinFunctions::ProhibitVoidMap )
-						$_ => $res->headers->header( $_ )
-					} $res->headers->header_field_names()
-				} );
-				$self->result_code( $response->reason_code() );
 		}
 		else {
 			$self->set_error_message( $response->reason_text() );
 		}
+
+		$self->order_number( $response->request_id() );
+		$self->response_code( $response->reason_code() );
+		$self->response_page( $res->content() );
+		$self->response_headers({
+				map { ## no critic ( BuiltinFunctions::ProhibitVoidMap )
+					$_ => $res->headers->header( $_ )
+				} $res->headers->header_field_names()
+			} );
+			$self->result_code( $response->reason_code() );
 	}
 	catch {
 		$message       = shift;
@@ -218,26 +222,26 @@ sub credit             {
 
 	try {
 		my $response      = $self->run_transaction( $request );
+		my $res         = $response->trace->response();
 
 		if ( $response->is_success() ) {
-			my $res         = $response->trace->response();
-
 			$success        = 1;
 
 			$self->is_success ( $success );
-			$self->order_number( $response->request_id() );
-			$self->response_code( $response->reason_code() );
-			$self->response_page( $res->content() );
-			$self->response_headers({
-					map { ## no critic ( BuiltinFunctions::ProhibitVoidMap )
-						$_ => $res->headers->header( $_ )
-					} $res->headers->header_field_names()
-				} );
-				$self->result_code( $response->reason_code() );
 		}
 		else {
 			$self->set_error_message( $response->reason_text() );
 		}
+
+		$self->order_number( $response->request_id() );
+		$self->response_code( $response->reason_code() );
+		$self->response_page( $res->content() );
+		$self->response_headers({
+				map { ## no critic ( BuiltinFunctions::ProhibitVoidMap )
+					$_ => $res->headers->header( $_ )
+				} $res->headers->header_field_names()
+			} );
+			$self->result_code( $response->reason_code() );
 	}
 	catch {
 		$message       = shift;
@@ -280,26 +284,26 @@ sub auth_reversal {
 
 	try {
 		my $response        = $self->run_transaction( $request );
+		my $res         = $response->trace->response();
 
 		if ( $response->is_success() ) {
-			my $res         = $response->trace->response();
-
 			$success        = 1;
 
 			$self->is_success ( $success );
-			$self->order_number( $response->request_id() );
-			$self->response_code( $response->reason_code() );
-			$self->response_page( $res->content() );
-			$self->response_headers({
-					map { ## no critic ( BuiltinFunctions::ProhibitVoidMap )
-						$_ => $res->headers->header( $_ )
-					} $res->headers->header_field_names()
-				} );
-				$self->result_code( $response->reason_code() );
 		}
 		else {
 			$self->set_error_message( $response->reason_text() );
 		}
+
+		$self->order_number( $response->request_id() );
+		$self->response_code( $response->reason_code() );
+		$self->response_page( $res->content() );
+		$self->response_headers({
+				map { ## no critic ( BuiltinFunctions::ProhibitVoidMap )
+					$_ => $res->headers->header( $_ )
+				} $res->headers->header_field_names()
+			} );
+			$self->result_code( $response->reason_code() );
 	}
 	catch {
 		$message       = shift;
@@ -626,7 +630,7 @@ Business::OnlinePayment::CyberSource::Client - CyberSource Client object  for Bu
 
 =head1 VERSION
 
-version 3.000004
+version 3.000005
 
 =head1 SYNOPSIS
 
